#========================================================================================
# package      DE_EPAGES::TrustedShops::t::unit::UI_Shop
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  unit test for UI::Shop
#========================================================================================
package DE_EPAGES::TrustedShops::t::unit::UI_Shop;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Test::Mock::Mockify qw ( GetParametersFromMockifyCall WasCalled );
use DE_EPAGES::Test::Mock::MockifyServlet;
use DE_EPAGES::TrustedShops::UI::Shop;
use Test::Exception;
use Test::MockObject::Extends;
use Test::More;

use feature 'switch';

use strict;

use constant TS_ID => 'X084304A3C16855B1ACB70DDBF66F37D7';

sub test {
    my $self = shift;
    $self->_test_SaveCertificates();
    $self->_test_SaveCertificates_FailedCreatingCertificateObject();

    done_testing();
}

#----------------------------------------------------------------------------------------
# Tests
#----------------------------------------------------------------------------------------
sub _test_SaveCertificates {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedUI_Shop = $self->_buildUIShop();
    my $MockedServlet = $self->_buildServlet();
    lives_ok( sub { $MockedUI_Shop->SaveCertificates($MockedServlet); }, "$SubTestName - SaveCertificates does not die." );
    my $TrustedShopsID = @{ GetParametersFromMockifyCall( $MockedUI_Shop, '_checkTSID' ) }[0];
    is( $TrustedShopsID, TS_ID, "$SubTestName - _checkTSID uses the right TrustedShopID." );
    $TrustedShopsID = @{ GetParametersFromMockifyCall( $MockedUI_Shop, '_checkTrustedShopsID' ) }[0];
    is( $TrustedShopsID, TS_ID, "$SubTestName - _checkTrustedShopsID uses the right TrustedShopID." );
    $TrustedShopsID = @{ GetParametersFromMockifyCall( $MockedUI_Shop, '_checkTrustedShopsIDExists' ) }[0];
    is( $TrustedShopsID, TS_ID, "$SubTestName - _checkTrustedShopsIDExists uses the right TrustedShopID." );
    ok( !WasCalled( $MockedUI_Shop, '_createFormError' ), "$SubTestName - createFormError was not executed." );

    return;
}

sub _test_SaveCertificates_FailedCreatingCertificateObject {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedUI_Shop = $self->_buildUIShop_WithFormError();
    my $MockedServlet = $self->_buildServlet();
    dies_ok( sub { $MockedUI_Shop->SaveCertificates($MockedServlet); }, "$SubTestName - SaveCertificates dies." );
    ok( WasCalled( $MockedUI_Shop, '_createFormError' ), "$SubTestName - createFormError was executed." );

    return;
}

#----------------------------------------------------------------------------------------
# Helpers
#----------------------------------------------------------------------------------------
sub _buildServlet {
    my $self = shift;

    return $self->_createServlet();
}

sub _buildUIShop {
    my $self = shift;

    my $hReturnValues = { 'createCertificateObject' => '18324', };

    $self->_createUIShop($hReturnValues);
}

sub _buildUIShop_WithFormError {
    my $self = shift;

    my $hReturnValues = { 'createCertificateObject' => undef, };

    $self->_createUIShop($hReturnValues);
}

sub _createForm {
    my $self = shift;

    my $MockedForm = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::WebInterface::API::Form');

    $MockedForm->mock(
        'executeFormError',
        sub {
            shift;
            my ($hFormError) = @_;

            my $ErrorReason = $hFormError->{'Reason'};
            Error($ErrorReason);

            return;
        }
    );

    $MockedForm->mock(
        'formValue',
        sub {
            shift;
            my ($InputName) = @_;

            given ($InputName) {
                when ('NewAlias') {
                    return TS_ID;
                }
                when ('IsExcellence') {
                    return 0;
                }
            }
        }
    );

    return $MockedForm;
}

sub _createServlet {
    my $self = shift;

    my $Shop           = $self->_createShop();
    my $MockifyServlet = DE_EPAGES::Test::Mock::MockifyServlet->new($Shop);

    $MockifyServlet->addMockWithReturnValue( 'form', $self->_createForm() );
    $MockifyServlet->addMockWithReturnValueAndParameterCheck( 'redirect', undef, ['string'] );

    return $MockifyServlet->getMockObject();
}

sub _createShop {
    my $self = shift;

    my $hAttributes = {};
    my $MockedShop = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Shop::API::Object::Shop', $hAttributes );

    $MockedShop->mock(
        'folder',
        sub {
            shift;
            my ($FolderName) = @_;

            if ( $FolderName eq 'TrustedShopsCertificates' ) {
                return $self->_getTrustedShopsCertificates();
            }

            return;
        }
    );

    return $MockedShop;
}

sub _createTrustedShopsCertificateHandler {
    my $self = shift;
    my ($hReturnValues) = @_;

    my $hAttributes = {
        'Shop'           => undef,
        'TrustedShopsID' => undef,
        'IsExcellence'   => undef,
    };
    my $MockedTrustedShopsCertificateHandler
      = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::TrustedShops::API::Object::TrustedShopsCertificateHandler', $hAttributes );

    $MockedTrustedShopsCertificateHandler->mock(
        'createCertificateObject',
        sub {
            shift;

            my $ObjectID = $hReturnValues->{'createCertificateObject'};
            if ( isValid($ObjectID) ) {
                return $ObjectID;
            }
            else {
                Error('ERROR');
            }
        }
    );

    return $MockedTrustedShopsCertificateHandler;
}

sub _createUIShop {
    my $self = shift;
    my ($hReturnValues) = @_;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::TrustedShops::UI::Shop');

    my $aParameterTypes = [ { 'object' => 'DE_EPAGES::Shop::API::Object::Shop' }, 'hashref', 'hashref', 'object' ];
    $Mockify->addMockWithReturnValueAndParameterCheck( '_BuildShopUrl', undef, $aParameterTypes );

    $aParameterTypes = [ { 'object' => 'DE_EPAGES::Shop::API::Object::Shop' }, 'string', 'int' ];
    $Mockify->addMockWithReturnValueAndParameterCheck(
        '_getTrustedShopsCertificateHandler',
        $self->_createTrustedShopsCertificateHandler($hReturnValues),
        $aParameterTypes
    );

    $aParameterTypes = [ 'string', 'any', 'string' ];
    $Mockify->addMockWithReturnValueAndParameterCheck( '_LogTrustedShops', undef, $aParameterTypes );

    $aParameterTypes = [ 'string', { 'object' => 'DE_EPAGES::WebInterface::API::Form' } ];
    $Mockify->addMethodSpyWithParameterCheck( '_checkTrustedShopsID', $aParameterTypes );

    $aParameterTypes = [ 'string', { 'object' => 'DE_EPAGES::WebInterface::API::Form' }, 'object' ];
    $Mockify->addMethodSpyWithParameterCheck( '_checkTrustedShopsIDExists', $aParameterTypes );

    $aParameterTypes = ['string'];
    $Mockify->addMethodSpyWithParameterCheck( '_checkTSID', $aParameterTypes );

    $aParameterTypes = [ { 'object' => 'DE_EPAGES::WebInterface::API::Form' }, 'string' ];
    $Mockify->addMethodSpyWithParameterCheck( '_createFormError', $aParameterTypes );

    return $Mockify->getMockObject();
}

sub _getTrustedShopsCertificates {
    my $self = shift;

    my $MockedObject = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::TrustedShops::API::Object::TrustedShopsCertificate');
    $MockedObject->mock(
        'existsChild',
        sub {
            shift;

            return 0;
        }
    );

    return $MockedObject;
}

__PACKAGE__->runScript();

1;
